"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = require("fs");

var _path = require("path");

var _utils = require("@fesjs/utils");

var _getModels = require("./utils/getModels");

var _getTmpFile = require("./utils/getTmpFile");

const namespace = 'plugin-model';

var _default = api => {
  const {
    paths,
    utils: {
      Mustache
    }
  } = api;

  function getModelDir() {
    return api.config.singular ? 'model' : 'models';
  }

  function getModelsPath() {
    return (0, _path.join)(paths.absSrcPath, getModelDir());
  }

  function getAllModels() {
    const srcModelsPath = getModelsPath();
    return _utils.lodash.uniq([...(0, _getModels.getModels)(srcModelsPath) // ...getModels(
    //     paths.absPagesPath,
    //     `**/${getModelDir()}/**/*.{js,jsx}`
    // ),
    // ...getModels(paths.absPagesPath, '**/*.model.{js,jsx}')
    ]);
  }

  const absCoreFilePath = (0, _path.join)(namespace, 'core.js');
  const absRuntimeFilePath = (0, _path.join)(namespace, 'runtime.js');
  const absInitlaStateFilePath = (0, _path.join)(namespace, 'models/initialState.js');
  api.register({
    key: 'addExtraModels',
    fn: () => [{
      absPath: (0, _utils.winPath)((0, _path.join)(paths.absTmpPath, absInitlaStateFilePath)),
      namespace: '@@initialState'
    }]
  });
  api.onGenerateFiles(async () => {
    const files = getAllModels();
    const additionalModels = await api.applyPlugins({
      key: 'addExtraModels',
      type: api.ApplyPluginsType.add,
      initialValue: []
    });
    const tmpFiles = (0, _getTmpFile.getTmpFile)(files, additionalModels, paths.absSrcPath);
    api.writeTmpFile({
      path: absCoreFilePath,
      content: Mustache.render((0, _fs.readFileSync)((0, _path.join)(__dirname, 'runtime/core.tpl'), 'utf-8'), { ...tmpFiles
      })
    });
    api.writeTmpFile({
      path: absRuntimeFilePath,
      content: Mustache.render((0, _fs.readFileSync)((0, _path.join)(__dirname, 'runtime/runtime.tpl'), 'utf-8'), {})
    });
    api.writeTmpFile({
      path: absInitlaStateFilePath,
      content: Mustache.render((0, _fs.readFileSync)((0, _path.join)(__dirname, 'runtime/models/initialState.tpl'), 'utf-8'), {})
    });
  });
  api.addPluginExports(() => [{
    specifiers: ['useModel'],
    source: absCoreFilePath
  }]);
  api.addRuntimePlugin(() => `@@/${absRuntimeFilePath}`);
};

exports.default = _default;